// --------------------------------------------------
// This code is used to solve the model BGG(1999):
// --------------------------------------------------

// ---------------------------------------
// --- claim the endogenous variables ----
// ---------------------------------------
var F G Ga Fp Gp Gap C Y Yf Pi Ce V K N I X P0 S1 S2 Q Rk R Rp omegabar D J A H;

varexo etar etaa;

// -----------------------------
// --- claim the parameters ----
// -----------------------------
parameters
// --- paramters ---
sig mu game bet xi zeta alp Omega del theta eps thetai gy rho phipi phiy rhoa

// - steady state
Fss Gss Gass Fpss Gpss Gapss Qss Piss Rss Rpss Rkss P0ss Xss Dss
 Hss Ass Kss Yss Iss Yfss Nss Vss Cess Css Mss Tss S1ss S2ss;

// --- calibration ---
sig     =    0.48;
mu      =    0.12;
bet     =    0.99;
zeta    =    3.33;
alp     =    0.35;
Omega   =    1 - 0.64/(1-alp);
del     =    0.025;
theta   =    0.75;
eps     =    3.85;
thetai  =    4.75;

gy      =    0.20;
rho     =    0.75;
phipi   =    1.5;
phiy    =    0.5/4;
rhoa    =    0.9;

// --- computing omegabarss -----
Jss     =    3/2;
param      = [sig mu Jss];
x0         = 0.4;
options    = optimoptions('fsolve', 'Display', 'iter');
[xx, fval] = fsolve(@(x) comp_ss_omegabar(x, param), x0, options);
omegabarss = xx;

Fss     =    normcdf((log(omegabarss))/sig + sig/2);
Gss     =    normcdf((log(omegabarss))/sig - sig/2);
Gass    =    Gss + omegabarss*(1-Fss);
Fpss    =    1/(sig*omegabarss)*normpdf((log(omegabarss))/sig + sig/2);
Gpss    =    1/(sig*omegabarss)*normpdf((log(omegabarss))/sig - sig/2);
Gapss   =    1 - Fss;

Qss     =    1;
Piss    =    (1+0.02)^(1/52);
Rss     =    1/bet;
Rpss    =    Piss*Rss;
Rkss    =    (Jss-1)/Jss*Rss/(Gass-mu*Gss);
P0ss    =    ((1-theta*Piss^(eps-1))/(1-theta))^(1/(1-eps));
Xss     =    eps/(eps-1)*(1-bet*theta*Piss^(eps-1))/(1-bet*theta*Piss^eps)*1/P0ss;
Dss     =    (1-theta)*P0ss^(-eps)/(1-theta*Piss^eps);

Hss     =    1/3;          // target moments for labor
Ass     =    1;
Kss     =    (alp*Ass/(Xss*(Rkss - 1 + del)))^(1/(1-alp)) * Hss^Omega;
Yss     =    Xss/alp*(Rkss - 1 + del)*Kss;
Iss     =    del*Kss;
Yfss    =    Yss/Dss;
Nss     =    Kss/Jss;
Vss     =   (1 - Gass)*Rkss*Kss;
game    =   Nss/Vss - (1-alp)*(1-Omega)*Yss/(Xss*Vss);
Cess    =   (1-game)*Vss;
Css     =   (1-gy)*Yfss - Cess - Iss - mu*Gss*Rkss*Kss;
xi      =   (1-alp)*Omega*(1-Hss)*Yss/(Xss*Hss*Css);
Mss     =   zeta*Css*Rpss/(Rpss - 1);
Tss     =   (1-gy)*Yfss - Mss + Mss/Piss;
S1ss    =   1/(1 - bet*theta*Piss^eps)*Yfss/(Xss*Css);
S2ss    =   1/(1 - bet*theta*Piss^(eps - 1))*Yfss/Css;

// -----------------------------
// ------ claim the model ------
// -----------------------------
model;
// --------------------
// - define variables -
// --------------------
F = normcdf(log(omegabar)/sig + sig/2);
G = normcdf(log(omegabar)/sig - sig/2);
Ga = G + omegabar*(1 - F);
Fp = 1/(sig*omegabar)*normpdf(log(omegabar)/sig + sig/2);
Gp = 1/(sig*omegabar)*normpdf(log(omegabar)/sig - sig/2);
Gap = 1 - F;

// -------------------------
// --- aggregate demand ----
// -------------------------
// equ(1): good market clearing
(1 - gy)*Yf = C + Ce + I + mu*G*Rk*Q(-1)*K(-1);

// equ(2): euler equation
bet*C/C(+1) = 1/R;

// equ(3): consumption of enterprenure
Ce = (1-game)*V;

// equ(4): zero profit condition
(Gap - mu * Gp)/Gap*(1 - Ga) + (Ga - mu*G) = R/Rk(+1);

// equ(5): capital demand
(Ga - mu*G)*Rk(+1)/R = (J - 1)/J;

// equ(6): leverage
J = Q * K/N;

// equ(7): capital return
Rk = alp*Y/(X*Q(-1)*K(-1)) + (1-del)*Q/Q(-1);

// equ(8): price of capital
Q = (thetai*(I/K(-1) - del) + 1)^(-1);

// -------------------------
// --- aggregate supply ----
// -------------------------
// equ(9): production
Y = A*K(-1)^alp*H^(Omega*(1-alp));

// equ(10): intermediate good
Y = D*Yf;

// equ(11): ;labor supply
xi*C/(1-H) = (1-alp)*Omega*Y/(X*H);

// equ(12): p0
P0 = eps/(eps-1)*S1/S2;

// equ(13): S1
S1 = Yf/(X*C) + bet*theta*Pi(+1)^eps*S1(+1);

// equ(14): S2
S2 = Yf/C + bet*theta*Pi(+1)^(eps-1)*S2(+1);

// equ(15): price index
theta*Pi^(eps-1) + (1-theta)*P0^(1-eps) = 1;

// equ(16): price disperse
D = theta*Pi^eps*D(-1) + (1-theta)*P0^(-eps);


// -------------------------
// ---- state variables ----
// -------------------------
// equ(17): accumulation of physical capital
K = (1-del)*K(-1) + I + thetai/2*(I/K(-1) - del)^2;

// equ(18): net worth accumulation
N = game * V + (1-alp)*(1-Omega)*Y/X;

// equ(19): value
V = (1-Ga)*Rk*Q(-1)*K(-1);

// equ(20): policy interest rate
Rp = R*Pi(+1);

// equ(21): monetary policy
log(Rp/Rpss) = rho*log(Rp(-1)/Rpss) + phipi*log(Pi/Piss) + etar;

// equ(22): exogenous variables
log(A) = rhoa*log(A(-1)) + etaa;
end;


// ---------------------------------------------------------
// initial steady state used in simulations
// ---------------------------------------------------------
initval;
F        =     Fss;
G        =     Gss;
Ga       =     Gass;
Fp       =     Fpss;
Gp       =     Gpss;
Gap      =     Gapss;
C        =     Css;
Y        =     Yss;
Yf       =     Yfss;
Pi       =     Piss;
Ce       =     Cess;
V        =     Vss;
K        =     Kss;
N        =     Nss;
I        =     Iss;
X        =     Xss;
P0       =     P0ss;
S1       =     S1ss;
S2       =     S2ss;
Q        =     Qss;
Rk       =     Rkss;
R        =     Rss;
Rp       =     Rpss;
omegabar =     omegabarss;
D        =     Dss;
J        =     Jss;
A        =     Ass;
H        =     Hss;
end;

// ------------------------------------------------------------------
//calculate residuals of dynamic equations with provided steady state
//and then invoke steady state computation followed by another check.
// ------------------------------------------------------------------
resid;
steady;
check;

shocks;
var etaa = 0.01^2;
var etar = 0.01^2;
end;

stoch_simul(irf = 40, order = 1, periods = 0, noprint, nograph) ;
%stoch_simul(irf = 40, order = 1, periods = 0, noprint, nograph) ;